﻿Option Strict On

Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Public Class LockBits
#Region "_declarations"
    Implements IDisposable
    Private sourceBMP As Bitmap = Nothing
    Private Iptr As IntPtr = IntPtr.Zero
    Private bitmapData As BitmapData = Nothing
    Private m_Pixels As Byte()
    Private m_Depth As Integer
    Private m_Width As Integer
    Private m_Height As Integer
#End Region
#Region "_properties"
    Public Property Pixels() As Byte()
        Get
            Return m_Pixels
        End Get
        Set(value As Byte())
            m_Pixels = value
        End Set
    End Property
    Public Property _BitmapData() As BitmapData
        Get
            Return bitmapData
        End Get
        Set(value As BitmapData)
            bitmapData = value
        End Set
    End Property
    Private Property Depth() As Integer
        Get
            Return m_Depth
        End Get
        Set(value As Integer)
            m_Depth = value
        End Set
    End Property
    Public Property Width() As Integer
        Get
            Return m_Width
        End Get
        Set(value As Integer)
            m_Width = value
        End Set
    End Property
    Public Property Height() As Integer
        Get
            Return m_Height
        End Get
        Set(value As Integer)
            m_Height = value
        End Set
    End Property
    Public ReadOnly Property Bitmap() As Bitmap
        Get
            Return sourceBMP
        End Get
    End Property
    Public Property _IntPtr As IntPtr
        Get
            Return Iptr
        End Get
        Set(value As IntPtr)
            Iptr = value
        End Set
    End Property
#End Region
#Region "_functions"
    Public Sub New(source As Bitmap)
        sourceBMP = source
    End Sub
    ''' <summary>
    ''' Lock bitmap data
    ''' </summary>
    Public Sub LockBits()
        Try
            ' Get width and height of bitmap
            Width = sourceBMP.Width
            Height = sourceBMP.Height

            ' get total locked pixels count
            Dim PixelCount As Integer = Width * Height
            ' Create rectangle to lock
            Dim rect As New Rectangle(0, 0, Width, Height)
            ' get source bitmap pixel format size
            Depth = Bitmap.GetPixelFormatSize(sourceBMP.PixelFormat)
            ' Check if bpp (Bits Per Pixel) is 8, 24, or 32
            If Depth <> 8 AndAlso Depth <> 24 AndAlso Depth <> 32 Then
                Throw New ArgumentException("Only 8, 24 and 32 bpp images are supported.")
            End If
            ' Lock bitmap and return bitmap data
            bitmapData = sourceBMP.LockBits(rect, ImageLockMode.ReadWrite, sourceBMP.PixelFormat)
            'Debug.WriteLine(bitmapData.Stride)
            ' create byte array to copy pixel values
            Dim [step] As Integer = Depth \ 8
            Pixels = New Byte(PixelCount * [step] - 1) {}
            Iptr = bitmapData.Scan0

            ' Copy data from pointer to array
            Marshal.Copy(Iptr, Pixels, 0, Pixels.Length)
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try
    End Sub
    ''' <summary>
    ''' Unlock bitmap data
    ''' </summary>
    Public Sub UnlockBits()
        Try
            ' Copy data from byte array to pointer
            Marshal.Copy(Pixels, 0, Iptr, Pixels.Length)

            ' Unlock bitmap data
            sourceBMP.UnlockBits(bitmapData)
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try
    End Sub
    ''' <summary>
    ''' Get the color of the specified pixel
    ''' </summary>
    ''' <param name="x"></param>
    ''' <param name="y"></param>
    ''' <returns></returns>
    Public Function GetPixel(x As Integer, y As Integer) As Color
        Dim clr As Color = Color.Empty

        ' Get color components count
        Dim cCount As Integer = Depth \ 8

        ' Get start index of the specified pixel
        Dim i As Integer = ((y * Width) + x) * cCount

        If i > Pixels.Length - cCount Then
            Throw New IndexOutOfRangeException()
        End If

        If Depth = 32 Then
            ' For 32 bpp get Red, Green, Blue and Alpha
            Dim b As Byte = Pixels(i)
            Dim g As Byte = Pixels(i + 1)
            Dim r As Byte = Pixels(i + 2)
            Dim a As Byte = Pixels(i + 3)
            ' a
            clr = Color.FromArgb(a, r, g, b)
        End If
        If Depth = 24 Then
            ' For 24 bpp get Red, Green and Blue
            Dim b As Byte = Pixels(i)
            Dim g As Byte = Pixels(i + 1)
            Dim r As Byte = Pixels(i + 2)
            clr = Color.FromArgb(r, g, b)
        End If
        If Depth = 8 Then
            ' For 8 bpp get color value (Red, Green and Blue values are the same)
            Dim c As Byte = Pixels(i)
            clr = Color.FromArgb(c, c, c)
        End If
        Return clr
    End Function
    ''' <summary>
    ''' Set the color of the specified pixel
    ''' </summary>
    ''' <param name="x"></param>
    ''' <param name="y"></param>
    ''' <param name="color"></param>
    Public Sub SetPixel(x As Integer, y As Integer, color As Color)
        ' Get color components count
        Dim cCount As Integer = Depth \ 8

        ' Get start index of the specified pixel
        Dim i As Integer = ((y * Width) + x) * cCount

        If Depth = 32 Then
            ' For 32 bpp set Red, Green, Blue and Alpha
            Pixels(i) = color.B
            Pixels(i + 1) = color.G
            Pixels(i + 2) = color.R
            Pixels(i + 3) = color.A
        End If
        If Depth = 24 Then
            ' For 24 bpp set Red, Green and Blue
            Pixels(i) = color.B
            Pixels(i + 1) = color.G
            Pixels(i + 2) = color.R
        End If
        If Depth = 8 Then
            ' For 8 bpp set color value (Red, Green and Blue values are the same)
            Pixels(i) = color.B
        End If
    End Sub
#End Region
#Region "IDisposable Support"
    Private disposedValue As Boolean ' So ermitteln Sie überflüssige Aufrufe
    ' IDisposable
    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' TODO: Verwalteten Zustand löschen (verwaltete Objekte).
            End If

            ' TODO: Nicht verwaltete Ressourcen (nicht verwaltete Objekte) freigeben und Finalize() unten überschreiben.
            ' TODO: Große Felder auf NULL festlegen.
        End If
        Me.disposedValue = True
    End Sub
    ' TODO: Finalize() nur überschreiben, wenn Dispose(ByVal disposing As Boolean) oben über Code zum Freigeben von nicht verwalteten Ressourcen verfügt.
    'Protected Overrides Sub Finalize()
    '    ' Ändern Sie diesen Code nicht. Fügen Sie oben in Dispose(ByVal disposing As Boolean) Bereinigungscode ein.
    '    Dispose(False)
    '    MyBase.Finalize()
    'End Sub
    ' Dieser Code wird von Visual Basic hinzugefügt, um das Dispose-Muster richtig zu implementieren.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Ändern Sie diesen Code nicht. Fügen Sie oben in Dispose(disposing As Boolean) Bereinigungscode ein.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region
End Class
